/* datatype.h - DMI - 23-8-2002

Copyright (C) 2001-02 David Ingram

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation. */

#include <stdlib.h>
#include <string.h>

typedef struct SCOP_StringBuf0
{
		char *buf; // Internal buffer - not null terminated
		int used, capacity;
		
} SCOP_StringBuf;

SCOP_StringBuf* SCOP_StringBuf_con();
void SCOP_StringBuf_des(SCOP_StringBuf*);
	
char* SCOP_StringBuf_compact(SCOP_StringBuf*);
		/* The string returned by compact() must be deleted by the caller,
			and is *not* removed when the StringBuf itself is deleted. */

void SCOP_StringBuf_cat(SCOP_StringBuf*, const char *s);
void SCOP_StringBuf_cat_c(SCOP_StringBuf*, char c);
		
void SCOP_StringBuf_clear(SCOP_StringBuf*);
int SCOP_StringBuf_length(SCOP_StringBuf*);
void SCOP_StringBuf_check_expand(SCOP_StringBuf*, int required);
		

/* svector's allocate storage for strings which are added, perform
	deep copies, and frees them when it is destroyed:
*/

typedef struct SCOP_svector0
{
		int capacity;
		int size;
		char **data;
		
} SCOP_svector;
			
SCOP_svector* SCOP_svector_con();
void SCOP_svector_des(SCOP_svector*);
		
void SCOP_svector_add(SCOP_svector*, char *s);
char* SCOP_svector_item(SCOP_svector*, int n);
void SCOP_svector_expand_capacity(SCOP_svector*);
		
int SCOP_svector_count(SCOP_svector*);
			

typedef struct SCOP_linefile0
{
		SCOP_svector *v;
		int file_ok;
	
} SCOP_linefile;

SCOP_linefile* SCOP_linefile_con(const char *filename);
void SCOP_linefile_des(SCOP_linefile*);
int SCOP_linefile_valid(SCOP_linefile*);
int SCOP_linefile_count(SCOP_linefile*);
		/* The valid() method (which tests the private file_ok flag set by
			the constructor) shows if the file could be opened. If not, the
			other methods will still work since v will exist anyway
			(but contain no items). */
		
const char *SCOP_linefile_getline(SCOP_linefile*, int n);
int SCOP_linefile_search(SCOP_linefile*, char *line);
	
int SCOP_linefile_readline(FILE *fp, char *buf, int max_len);		
